from django.contrib.auth.mixins import LoginRequiredMixin

class OwnedObjectQuerysetMixin(LoginRequiredMixin):
    """
    CBV mixin which filters the queryset based on owner.

    Assumes that the model is an OwnedModel, or at least that it has an owner
    foreign key to the auth user.
    """
    def get_queryset(self):
        queryset = super(OwnedObjectQuerysetMixin, self).get_queryset()
        return queryset.filter(owner=self.request.user)


class OwnedObjectFormMixin(LoginRequiredMixin):
    """
    CBV mixin that adds the logged in user's instance to the created object.
    """
    def form_valid(self, form):
        form.instance.owner = self.request.user
        return super(OwnedObjectFormMixin, self).form_valid(form)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-15 11:56
from __future__ import unicode_literals

from django.db import migrations, models

from test_app.models import TestModel


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmodel',
            name='test2',
            field=models.CharField(default='', help_text='', max_length=255),
            preserve_default=False,
        ),
    ]



from django.db import connections
from django.db.models.signals import post_init
from django.db.migrations.state import AppConfigStub

def meta_equal(m1, m2):
    return (m1.app_config.name == m2.app_config.name and
            m1.model_name == m2.model_name)

# TODO: Handle inheritance
def is_model_instance(instance, model):
    # Don't trigger on half-migrated objects
    if isinstance(instance._meta.app_config, AppConfigStub):
        return False

    return (isinstance(instance, model) or meta_equal(instance._meta,
                                                      model._meta))


class MigrationHandler(object):
    # TODO: Figure out how to handle multiple dbs
    def __init__(self):
        self.migration_handler_plans = {}
        post_init.connect(self.migration_handler_pre_post)

    def migration_handler_pre_post(self, sender, **kwargs):
        from django.db.migrations.recorder import MigrationRecorder
        from django.db.migrations.executor import MigrationExecutor

        self.migration_class = MigrationRecorder.Migration
        self.executors = {}
        self.plans = {}
        self.sender_plans = {}

        for db in connections:
            connection = connections[db]
            executor = MigrationExecutor(connection)
            self.executors[db] = executor
            targets = executor.loader.graph.leaf_nodes()
            self.plans[db] = executor.migration_plan(targets)

        post_init.connect(self.migration_handler_post)
        post_init.disconnect(self.migration_handler_pre_post)

        self.migration_handler_post(sender, **kwargs)

    # TODO: Provide a means of blanket or selective clearing of the plan cache
    def migration_handler_post(self, sender, instance, **kwargs):
        if sender == self.migration_class:
            return

        plan = self.migration_handler_plans.get(sender)

        if plan is not None:
            for handler, apps in plan:
                handler(instance, apps)
        else:
            plan = []
            for migration, backwards in self.plans['default']:
                model_handlers = getattr(migration, 'model_handlers', [])
                for model, handler in model_handlers:
                    if is_model_instance(instance, model):
                        executor = self.executors['default']
                        migration_state = [(migration.app_label,
                                            migration.name)]
                        apps = executor.loader.project_state(migration_state).apps
                        handler(instance, apps)
                        plan.append((handler, apps))

            self.migration_handler_plans[sender] = plan

handler = MigrationHandler()



from ldb.django.test.testcases import DjangoAuthAppTestCaseMixin
from django.test.testcases import TransactionTestCase

class ModelTestCase(DjangoAuthAppTestCaseMixin, TransactionTestCase):
    applications = ['test_app', 'ldb.django.contrib.migration']
    root_urlconf = ''

    def test_a(self):
        from test_app.models import TestModel

        a = TestModel(test='asdf', test3='test3')
        a.save()
        a_pk = a.pk
        self.assertEqual(TestModel.objects.get(pk=a_pk).test, 'asdf')



from django.contrib.auth.models import User
from django.test import Client
from ldb.django.test import SeleniumTestCase
from django.conf import settings

import urlparse

class UserTestMixin(object):
    """
    Mixin that handles test that require a user.
    
    Does the following:
        * Creates a django.test.Client and stores it in cls.client.
        * Creates a user, stores user in cls.user
        * Logs in said user to cls.client
    """
    username = 'test_user'
    password = 'silly_pass'

    def _pre_setup(self):
        super(UserTestMixin, self)._pre_setup()
        self.client = Client()
        self.user = User.objects.create_user(self.username, 'test@user.com',
                                             self.password)
        self.user.save()
        self.client.login(username=self.username, password=self.password)


class SeleniumUserTestCase(UserTestMixin, SeleniumTestCase):
    # TODO: Handle exceptions, especially if login fails
    def _pre_setup(self):
        super(SeleniumUserTestCase, self)._pre_setup()
        settings.DEBUG = True
        self.selenium_get(settings.LOGIN_URL)
        self.selenium.find_element_by_name('username').send_keys(self.username)
        password_field = self.selenium.find_element_by_name('password')
        password_field.send_keys(self.password)
        password_field.submit()

        expected_url = self.make_url(settings.LOGIN_REDIRECT_URL)
        if self.selenium.current_url != expected_url:
            super(SeleniumUserTestCase, self)._post_teardown()
            assert False, "Login failed"



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-15 18:30
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0003_move_model_update'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='testmodel',
            name='test2',
        ),
        migrations.AlterField(
            model_name='testmodel',
            name='test3',
            field=models.CharField(help_text='', max_length=255),
        ),
    ]



from django.test.utils import override_settings
from django.test import LiveServerTestCase

from django.conf import global_settings

import tempfile
import shutil
import urlparse

class MediaSandboxTestMixin(object):
    # TODO: Handle exceptions (call teardown or w/e needs to happen)
    @classmethod
    def setUpClass(cls):
        cls._media_root = tempfile.mkdtemp()
        cls._media_context = override_settings(MEDIA_ROOT=cls._media_root)
        cls._media_context.enable()
        super(MediaSandboxTestMixin, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(MediaSandboxTestMixin, cls).tearDownClass()
        cls._media_context.disable()
        shutil.rmtree(cls._media_root)


class SeleniumTestCase(LiveServerTestCase):
    # TODO: Handle exceptions (call teardown or w/e needs to happen)
    @classmethod
    def setUpClass(cls):
        from selenium import webdriver

        super(SeleniumTestCase, cls).setUpClass()
        cls.selenium = webdriver.Firefox()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(SeleniumTestCase, cls).tearDownClass()

    @classmethod
    def make_url(cls, rel_address):
        return urlparse.urljoin(cls.live_server_url, rel_address)

    @classmethod
    def selenium_get(cls, rel_address):
        cls.selenium.get(cls.make_url(rel_address))


class DjangoAppTestCaseMixin(object):
    applications = []
    pre_middleware = []
    post_middleware = []

    @classmethod
    def setUpClass(cls):
        settings = {'DEBUG': True,
                    'SECRET_KEY': 'fake_key',
                    'INSTALLED_APPS': cls.get_applications(),
                    'ROOT_URLCONF': cls.root_urlconf,
                    'STATIC_URL': '/static/',
                    'MEDIA_ROOT': None,
                    'LOGGING': {
                        'version': 1,
                        'disable_existing_loggers': True,
                        'handlers': {
                            'console': {
                                'level': 'INFO',
                                'class': 'logging.StreamHandler'
                            }
                        },
                        'loggers': {
                            'django.request': {
                                'handlers': ['console'],
                                'propogate': True
                            }
                        }
                    },
                    'MIDDLEWARE_CLASSES': cls.get_middleware()
                   }
        cls.__settings_context = override_settings(**settings)
        cls.__settings_context.enable()

        super(DjangoAppTestCaseMixin, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(DjangoAppTestCaseMixin, cls).tearDownClass()

        cls.__settings_context.disable()

    @classmethod
    def get_applications(cls):
        return global_settings.INSTALLED_APPS + list(cls.applications)

    @classmethod
    def get_middleware(cls):
        return (list(cls.pre_middleware) + global_settings.MIDDLEWARE_CLASSES +
                list(cls.post_middleware))


class DjangoSessionAppTestCaseMixin(DjangoAppTestCaseMixin):
    @classmethod
    def get_applications(cls):
        return (super(DjangoSessionAppTestCaseMixin, cls).get_applications() + 
                ['django.contrib.sessions'])

    @classmethod
    def get_middleware(cls):
        return (super(DjangoSessionAppTestCaseMixin, cls).get_middleware() + 
                ['django.contrib.sessions.middleware.SessionMiddleware'])


class DjangoContentTypeAppTestCaseMixin(DjangoAppTestCaseMixin):
    @classmethod
    def get_applications(cls):
        return (super(DjangoContentTypeAppTestCaseMixin,
                      cls).get_applications() + 
                ['django.contrib.contenttypes'])


class DjangoAuthAppTestCaseMixin(DjangoSessionAppTestCaseMixin,
                                 DjangoContentTypeAppTestCaseMixin):
    @classmethod
    def get_applications(cls):
        return (super(DjangoAuthAppTestCaseMixin, cls).get_applications() + 
                ['django.contrib.auth'])

    @classmethod
    def get_middleware(cls):
        return (super(DjangoAuthAppTestCaseMixin, cls).get_middleware() + 
                ['django.contrib.auth.middleware.AuthenticationMiddleware'])



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-15 18:30
from __future__ import unicode_literals

from django.db import migrations, models
from test_app.models import TestModel


def migrate_testmodel(apps, schema_editor):
    TestModel = apps.get_model('test_app', 'TestModel')
    for test_model in TestModel.objects.all():
        # Have to do this by hand because we asked for the model at this state
        # opposed to the "full" model
        test_model.test3 = test_model.test2
        test_model.save()


def unmigrate_testmodel(apps, schema_editor):
    TestModel = apps.get_model('test_app', 'TestModel')
    for test_model in TestModel.objects.all():
        test_model.test2 = test_model.test3
        test_model.save()


def handle_testmodel(inst, apps):
    if not hasattr(inst, 'test3'):
        old_inst = apps.get_model('test_app',
                                  'TestModel').objects.get(pk=inst.pk)
        inst.test3 = old_inst.test2


class Migration(migrations.Migration):
    model_handlers = [
        (TestModel, handle_testmodel),
    ]

    dependencies = [
        ('test_app', '0003_move_model_add_test3'),
    ]

    operations = [
        migrations.RunPython(migrate_testmodel, unmigrate_testmodel),
    ]
